@@ -6,6 +6,8 @@ import json
 import boto3
 from sqlalchemy import create_engine, and_
 from sqlalchemy.orm import sessionmaker, undefer
+from sqlalchemy.orm.attributes import flag_modified
+
 import sdc_helpers.utils as utils
 import sdc_helpers.decorators as decorators
 from sdc_helpers.models.client import Client
@@ -173,7 +175,13 @@ class QueryHelper:
                     value=json.dumps(client.to_dict(), default=str)
                 )
         else:
-            client = Client(**json.loads(client_redis))
+            client_dict = json.loads(client_redis)
+            client_dict.update(
+                {
+                    'read_only': True
+                }
+            )
+            client = Client(**client_dict)
 
         return client
 
@@ -223,7 +231,13 @@ class QueryHelper:
                     value=json.dumps(service.to_dict(), default=str)
                 )
         else:
-            service = Service(**json.loads(service_redis))
+            service_dict = json.loads(service_redis)
+            service_dict.update(
+                {
+                    'read_only': True
+                }
+            )
+            service = Service(**service_dict)
 
         return service
 
@@ -275,7 +289,13 @@ class QueryHelper:
                     value=json.dumps(subscription.to_dict(), default=str)
                 )
         else:
-            subscription = Subscription(**json.loads(subscription_redis))
+            subscription_dict = json.loads(subscription_redis)
+            subscription_dict.update(
+                {
+                    'read_only': True
+                }
+            )
+            subscription = Subscription(**subscription_dict)
 
         return subscription
 
@@ -303,21 +323,19 @@ class QueryHelper:
     @decorators.query_exception_handler()
     def update_subscription(self, *, subscription: Subscription):
         """
-            Commit the Subscription model and refresh Redis for the subscription
+            Commit the Subscription model and flush all subscription Redis keys
 
             args:
                 subscription (Subscription): The Subscription model to update
         """
+        if subscription.read_only:
+            raise Exception('ServerError: Cannot update a read only model')
+
+        # If properties have only been partially modified,
+        # e.g `subscription.properties['node1'][0][node2] = value`,
+        # the properties attribute is not marked as dirty and we
+        # need to mark it as such to cause an update in the database
+
+        flag_modified(subscription, 'properties')
+
         self.session.commit()
-
-        subscription_redis_key = (
-            'subscription-properties-{client_id}-{service_id}'.format(
-                client_id=subscription.client_id,
-                service_id=subscription.service_id
-            )
-        )
-
-        self.redis_helper.redis_set(
-            key=subscription_redis_key,
-            value=json.dumps(subscription.to_dict(), default=str)
-        )
